unit Main_U;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Spin, ExtCtrls, StdCtrls;

type
  TMain_F = class(TForm)
    OpenDialog1: TOpenDialog;
    edVolFilename: TEdit;
    pbBrowse1: TButton;
    Label1: TLabel;
    rbRandomOptions: TGroupBox;
    rbPseudoRandomData: TRadioButton;
    rbRandomFromFile: TRadioButton;
    edRandomFilename: TEdit;
    Label2: TLabel;
    pbBrowse2: TButton;
    rgOperation: TRadioGroup;
    pbModify: TButton;
    pbClose: TButton;
    Label4: TLabel;
    speRemoveBytes: TSpinEdit;
    Label5: TLabel;
    Label3: TLabel;
    lblVersion: TLabel;
    pbComputeCountLastBytes: TButton;
    procedure pbBrowse1Click(Sender: TObject);
    procedure pbBrowse2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pbCloseClick(Sender: TObject);
    procedure pbModifyClick(Sender: TObject);
    procedure rgOperationClick(Sender: TObject);
    procedure pbComputeCountLastBytesClick(Sender: TObject);
  private
    procedure TruncateFile();
    procedure RandomFillFile();

    function  doit(driveLetter: char; filename: string): integer;
    procedure FillFreeDiskSpace(junkDir: string; fillByte: byte);
    function  CountZerosInFile(filename: string): DWORD;
    procedure EraseJunkFiles(junkDir: string);
  public
    { Public declarations }
  end;

var
  Main_F: TMain_F;

implementation

{$R *.DFM}

uses
  SDUGeneral,
  SDUFileIterator_U,
  MountsAs_U;

const CLUSTER_SIZE = 1024; // For max efficiency, this should be set to the
                           // cluster size of the mounted seNTry 2020 volume

procedure TMain_F.pbBrowse1Click(Sender: TObject);
begin
  opendialog1.filterindex := 1;
  SDUOpenSaveDialogSetup(opendialog1, edVolFilename.text);
  if opendialog1.execute() then
    begin
    edVolFilename.text := opendialog1.filename;
    end;

end;

procedure TMain_F.pbBrowse2Click(Sender: TObject);
begin
  opendialog1.filterindex := 2;
  SDUOpenSaveDialogSetup(opendialog1, edRandomFilename.text);
  if opendialog1.execute() then
    begin
    edRandomFilename.text := opendialog1.filename;
    end;

end;

procedure TMain_F.FormCreate(Sender: TObject);
var
  majorVersion, minorVersion, releaseVersion, buildVersion: integer;
begin
  speRemoveBytes.maxvalue := high(integer);
  speRemoveBytes.value := $E00;

  SDUGetVersionInfo('', majorVersion, minorVersion, releaseVersion, buildVersion);
  lblVersion.caption := Format('v%d.%.2d.%.2d.%.4d', [majorVersion, minorVersion, releaseVersion, buildVersion]);

end;

procedure TMain_F.pbCloseClick(Sender: TObject);
begin
  Close;

end;

procedure TMain_F.pbModifyClick(Sender: TObject);
begin
  if not(FileExists(edVolFilename.text)) then
    begin
    showmessage('Volume file "'+edVolFilename.text+'" does not exist');
    exit;
    end;

  if rgOperation.itemindex = 1 then
    begin
    TruncateFile();
    end
  else
    begin
    RandomFillFile();
    end;

  showmessage('Finished.');

end;

procedure TMain_F.TruncateFile();
var
  fileHandle : THandle;
begin
  if MessageDlg('You have opted to truncate the last '+
                 speRemoveBytes.text+
                 ' bytes from file:'+
                 #13+#10+
                 edVolFilename.text+
                 #13+#10+
                 #13+#10+
                 'Are you sure you wish to do this?',
                mtConfirmation,
                [mbYes, mbNo],
                0) = mrYes then
    begin
    fileHandle := CreateFile(PChar(edVolFilename.text),
                             GENERIC_READ or GENERIC_WRITE,
                             0,
                             nil,
                             OPEN_EXISTING,
                             FILE_ATTRIBUTE_NORMAL,
                             0);
    if fileHandle=INVALID_HANDLE_VALUE then
      begin
      showmessage('Unable to open volume file:'+#13+#10+edVolFilename.text+#13+#10+#13+#10+'Please ensure that this filename is correct that this volume is not mounted');
      exit;
      end;

    SetFilePointer(fileHandle, -speRemoveBytes.value, nil, FILE_END);
    SetEndOfFile(fileHandle);

    CloseHandle(fileHandle);
    end;

end;

procedure TMain_F.RandomFillFile();
var
  strSelected: string;
  dataBlock: array of byte;
  fileHandle: THandle;
  i: integer;
  bytesWritten: DWORD;
  dataFileSizeLo: DWORD;
  dataFileSizeHi: DWORD;
begin
  if not(FileExists(edVolFilename.text)) then
    begin
    showmessage('Volume file "'+edVolFilename.text+'" does not exist');
    exit;
    end;

  if rbPseudoRandomData.checked then
    begin
    strSelected := 'with pseudorandom data';
    end
  else
    begin
    strSelected := 'with the first '+speRemoveBytes.text+' bytes from file:'+#13+#10+edRandomFilename.text;
    end;

  if MessageDlg('You have opted to replace the last '+
                 speRemoveBytes.text+
                 ' bytes from file:'+
                 #13+#10+
                 edVolFilename.text+
                 #13+#10+
                 strSelected+
                 #13+#10+
                 #13+#10+
                 'Are you sure you wish to do this?',
                mtConfirmation,
                [mbYes, mbNo],
                0) = mrYes then
    begin
    SetLength(dataBlock, speRemoveBytes.value);

    if rbPseudoRandomData.checked then
      begin
      Randomize;
      for i:=0 to speRemoveBytes.value-1 do
        begin
        dataBlock[i] := random(255);
        end;
      end
    else
      begin
      // Check that the random data file (if needed) contains enough data
      fileHandle := CreateFile(PChar(edRandomFilename.text),
                               GENERIC_READ,
                               0,
                               nil,
                               OPEN_EXISTING,
                               FILE_ATTRIBUTE_NORMAL,
                               0);

      if fileHandle=INVALID_HANDLE_VALUE then
        begin
        showmessage('Unable to open file containing random data:'+#13+#10+edRandomFilename.text);
        exit;
        end;

      dataFileSizeLo := GetFileSize(fileHandle, @dataFileSizeHi);
      if (dataFileSizeLo<DWORD(speRemoveBytes.value)) AND (dataFileSizeHi<1) then
        begin
        showmessage('File containing random data too small. Must be at least '+speRemoveBytes.text+' bytes');
        CloseHandle(fileHandle);
        exit;
        end;

      SetFilePointer(fileHandle, 0, nil, FILE_BEGIN);

      ReadFile(fileHandle, dataBlock[0], speRemoveBytes.value, bytesWritten, nil);
      CloseHandle(fileHandle);
      end;

    // Write out speRemoveBytes.value to the file.
    fileHandle := CreateFile(PChar(edVolFilename.text),
                             GENERIC_READ or GENERIC_WRITE,
                             0,
                             nil,
                             OPEN_EXISTING,
                             FILE_ATTRIBUTE_NORMAL,
                             0);
    if fileHandle=INVALID_HANDLE_VALUE then
      begin
      showmessage('Unable to open volume file:'+#13+#10+edVolFilename.text+#13+#10+#13+#10+'Please ensure that this filename is correct that this volume is not mounted');
      exit;
      end;

    SetFilePointer(fileHandle, -speRemoveBytes.value, nil, FILE_END);

    WriteFile(fileHandle, dataBlock[0], speRemoveBytes.value, bytesWritten, nil);
    CloseHandle(fileHandle);
    end;

end;

procedure TMain_F.rgOperationClick(Sender: TObject);
begin
  SDUEnableControl(rbRandomOptions, (rgOperation.itemindex <> 1));

end;

procedure TMain_F.pbComputeCountLastBytesClick(Sender: TObject);
var
  mntAs: TMountsAs_F;
begin
  mntAs:= TMountsAs_F.Create(nil);
  try
    mntAs.lblVolFilename.caption := edVolFilename.text;

    if mntAs.ShowModal()=mrOK then
      begin
      speRemoveBytes.value := doit(mntAs.cbDriveLetter.text[1], edVolFilename.text);
      showmessage(speRemoveBytes.text+' bytes at the end of this volume file appear to be inaccessable to the mounted volume');
      end;

  finally
    mntAs.Free();
  end;

end;

function TMain_F.doit(driveLetter: char; filename: string): integer;
var
  currFillByte: byte;
  lastZeroInFilePos: DWORD;
  currZeroInFilePos: DWORD;
  junkDir: string;
  finished: boolean;
begin
  driveLetter := upcase(driveLetter);

  // This should be worked out properly as a nice random junk dir, which
  // doesn't already exist, but this will do for now...
  junkDir:= driveLetter+':\JUNKDIR';

  currFillByte := $AA; // Alternating 10101010
  lastZeroInFilePos := 0;
  finished:=FALSE;
  showmessage('Please MOUNT the volume'+#13+#10+filename+#13+#10+'as drive '+driveLetter+': and click OK');

  if not(CreateDir(junkDir)) then
    begin
    showmessage('Unable to create temp dir "'+junkdir+'"');
    finished := TRUE;
    end;

  while not(finished) do
    begin
    currFillByte := not(currFillByte);

    // now fill the disk
    FillFreeDiskSpace(junkDir, currFillByte);
    showmessage('Please DISMOUNT the volume'+#13+#10+filename+#13+#10+'currently mounted as drive '+driveLetter+': and click OK');

    // note where the last 0 in the .raw file is
    currZeroInFilePos := CountZerosInFile(filename);

    showmessage('Please MOUNT the volume'+#13+#10+filename+#13+#10+'as drive '+driveLetter+': and click OK');
    EraseJunkFiles(junkDir);

    if lastZeroInFilePos=currZeroInFilePos then
      begin
      finished := TRUE;
      end;

    lastZeroInFilePos:=currZeroInFilePos;

    end;

  RemoveDir(junkDir);
  Result := lastZeroInFilePos;

end;

procedure TMain_F.FillFreeDiskSpace(junkDir: string; fillByte: byte);
var
  buffer: array [0..(CLUSTER_SIZE-1)] of byte;
  i: integer;
  bytesWritten: DWORD;
  fileHandle: THandle;
  fileNo: integer;
  currFilename: string;
  finished: boolean;
begin
  for i:=low(buffer) to high(buffer) do
    begin
    buffer[i] := fillByte;
    end;

  fileNo:= 0;

  finished := FALSE;
  while not(finished) do
    begin
    inc(fileNo);
    currFilename := junkDir + '\padding_'+inttostr(fileNo);

    fileHandle := CreateFile(PChar(currFilename),
                             GENERIC_READ or GENERIC_WRITE,
                             0,
                             nil,
                             CREATE_ALWAYS,
                             FILE_ATTRIBUTE_NORMAL,
                             0);

//    for i:=1 to 10240 do // 10MB
    while not(finished) do
      begin
      WriteFile(fileHandle, buffer, sizeof(buffer), bytesWritten, nil);
      if bytesWritten<sizeof(buffer) then
        begin
        finished := TRUE;
        break;
        end;
      end; // for i:=1 to 1024 do

    CloseHandle(fileHandle);

    end; // while not(finished) do


end;


function TMain_F.CountZerosInFile(filename: string): DWORD;
var
  count: integer;
  lastByte: array [0..1] of byte;
  bytesRead: DWORD;
  fileHandle: THandle;
begin
  Result := 0;

  fileHandle := CreateFile(PChar(filename),
                           GENERIC_READ,
                           0,
                           nil,
                           OPEN_EXISTING,
                           FILE_ATTRIBUTE_NORMAL,
                           0);

  if fileHandle=INVALID_HANDLE_VALUE then
    begin
    showmessage('Unable to open volume file:'+#13+#10+edVolFilename.text+#13+#10+#13+#10+'Please ensure that this filename is correct that this volume is not mounted');
    exit;
    end;

  count := 0;
  lastByte[0]:=0;
  while (lastByte[0]=0) do
    begin
    inc(count);
    SetFilePointer(fileHandle, -count, nil, FILE_END);
    ReadFile(fileHandle, lastByte[0], 1, bytesRead, nil);
    end;

  dec(count);
  CloseHandle(fileHandle);

  Result := count;

end;


procedure TMain_F.EraseJunkFiles(junkDir: string);
var
  fileIterator: TSDUFileIterator;
  currFilename: string;
begin
  fileIterator:= TSDUFileIterator.create(nil);
  try
    fileIterator.Directory := junkDir;
    fileIterator.Reset();
    currFilename:=fileIterator.Next();
    while (currFilename<>'') do
      begin
      DeleteFile(currFilename);
      currFilename:=fileIterator.Next();
      end;

  finally
    fileIterator.Free();
  end;

end;

END.

